"""droplo - Official Python SDK for the droplo.io content API"""

__title__ = 'droplo'
__version__ = '0.1.1'
__author__ = 'Droplo Inc <contact@droplo.io>'
__license__ = 'MIT'

from .api import Api  # noqa: F401



"""
droplo.resource
~~~~~~~~~~~~~~~~~~~
This module implements the Resource, FolderResource and FileResource classes.
"""


class Resource(object):
    """
    Base Resource Class
    Implements common resource attributes.
    """
    def __init__(self, data):
        self.raw = data
        self.name = data.get('name')
        self.created = data.get('created')
        self.modified = data.get('modified')


class FolderResource(Resource):
    """Folder Resource Class
    """
    def __init__(self, data):
        super(FolderResource, self).__init__(data)
        self.endpoint = data.get('endpoint')


class FileResource(Resource):
    """File Resource Class
    """
    def __init__(self, data):
        super(FileResource, self).__init__(data)
        self.type = data.get('type')
        self.url = data.get('url')



"""
droplo.exceptions
~~~~~~~~~~~~~~~~~~~

This module contains the set of droplo exceptions.
"""


class Error(Exception):
    """Base class for exceptions in this module."""
    def __str__(self):
        return self.__doc__


class InvalidURLError(Error):
    """The endpoint URL is invalid"""
    pass


class InvalidTokenError(Error):
    """The provided access token is either invalid or expired"""
    pass


class AuthorizationNeededError(Error):
    """You need to provide an access token to access this repository"""
    pass


class HTTPError(Error):
    """HTTP error"""

    def __init__(self, code, message):
        self.code = code
        self.message = message
        super(HTTPError, self).__init__()

    def __str__(self):
        return ("Got an HTTP error %(code)d (%(message)s)" %
                {"code": self.code, "message": self.message})


class UnexpectedError(Error):
    """Unexpected error."""

    def __init__(self, message):
        self.message = message
        super(UnexpectedError, self).__init__()

    def __str__(self):
        return self.message


class RefMissing(Error):
    """You need to provide the ref parameter."""
    pass



from droplo.cache import ShelveCache
import unittest
import time


class TestCache(unittest.TestCase):

    def setUp(self):
        self.cache = ShelveCache("cachetest")

    def test_cache_set_get(self):
        self.cache.set("foo", "bar", 3600)
        self.assertEqual(self.cache.get("foo"), "bar")

    def test_cache_expiration(self):
        self.cache.set("toto", "tata", 2)
        time.sleep(3)
        self.assertIsNone(self.cache.get("toto"))


if __name__ == '__main__':
    unittest.main()



"""
droplo.api
~~~~~~~~~~~~~~~~~

This module implements the Droplo content API Client,
allowing interaction with every method present in it.
"""

from .connection import get_json
from .resource import FolderResource, FileResource


class Api(object):
    """Constructs the API Client.

    :param access_token: API Access Token.

    Usage:

        >>> import droplo
        >>> api = droplo.Api('YOUR_ACCESS_TOKEN')
        <droplo.Api access_token="b4c0n73n7fu1">
    """

    def __init__(
        self,
        access_token,
        api_url='https://api.droplo.io',
        cache=None
    ):
        self.access_token = access_token
        self.api_url = api_url
        self.ttl = 10
        self.cache = cache

    def get(self, path='/'):
        """Fetches an api endpoint.

        :param path: (optional) The path of the endpoint.
        :return: :class:`Response <droplo.api.Response>` object.
        :rtype: droplo.api.Response

        Usage:
            >>> entries = api.get('blogposts')
            [<Entry[File] name='My First Blogpost' uuid="73f395ba-0478-4742-aeb7-f71a92f195ca">,
             <Entry[File] name='My Second Blogpost' uuid="7ced093f-436f-4935-89a4-9c86b84de3a3">]
        """

        return Response(get_json(
            self.build_url(path),
            self.access_token,
            cache=self.cache,
            ttl=self.ttl
        ))

    def build_url(self, path):
        endpoint = path[1:] if path.startswith('/') else path
        return '{0}/{1}'.format(self.api_url, endpoint)

    def __repr__(self):
        return '<droplo.api.Api access_token="{0}" api_url="{1}">'.format(self.access_token, self.api_url)


class Response(object):
    """
    Droplo's response to a query.
    :ivar list<droplo.resource.FolderResource> folders: the folders in the current path
    :ivar list<droplo.resource.FileResource> files: the files in the current path
    """

    def __init__(self, data):
        self.data = data
        self.folders = [FolderResource(d) for d in data.get("folders", [])]
        self.files = [FileResource(d) for d in data.get("files", [])]

    def __repr__(self):
        return '<droplo.api.Response>'



"""
droplo.connection
~~~~~~~~~~~

This module implements the Droplo Connection handlers.

"""

import requests
import platform
import re
from . import __version__ as droplo_version
from .cache import ShelveCache
from .exceptions import (InvalidTokenError, AuthorizationNeededError, HTTPError)


def get_json(url, access_token='', cache=None, ttl=None):
    if cache is None:
        cache = ShelveCache(access_token)
    cached = cache.get(url)
    if cached is not None:
        return cached

    response = requests.get(url, headers={
        "Authorization": "Bearer {}".format(access_token),
        "User-Agent": "droplo-python-client/%s Python/%s" % (
            droplo_version,
            platform.python_version()
        )
    })
    json_result = response.json()

    if response.status_code == 200:
        expire = ttl or get_max_age(response.headers)
        if expire is not None:
            cache.set(url, json_result, expire)
        return json_result
    elif response.status_code == 401:
        if len(access_token) == 0:
            raise AuthorizationNeededError()
        else:
            raise InvalidTokenError()
    else:
        raise HTTPError(response.status_code, str(json_result['message']))


def get_max_age(headers):
    expire_header = headers.get("Cache-Control", None)
    if expire_header is not None:
        m = re.match("max-age=(\d+)", expire_header)
        if m:
            return int(m.group(1))
    return None



import droplo
import unittest


class ApiTestCase(unittest.TestCase):
    def setUp(self):
        self.url = "https://api.dev.droplo.io"
        self.token = "c471fb87d07eee1e16c34e5d47d2e721ea141e7d05b1092938e5ff943b0bd09f"

    def tearDown(self):
        pass

    def test_api_missing_accesstoken(self):
        with self.assertRaises(TypeError):
            droplo.Api()

    def test_api_endpoints(self):
        api = droplo.Api(self.token, api_url=self.url)
        endpoints = api.get()
        self.assertEqual(endpoints.folders[0].endpoint, '{0}/blogposts'.format(self.url))

    def test_api_files(self):
        api = droplo.Api(self.token, api_url=self.url)
        endpoints = api.get('/blogposts')
        self.assertEqual(endpoints.files[0].type, 'image/jpeg')


if __name__ == '__main__':
    unittest.main()



"""
droplo.cache
~~~~~~~~~~~~~~~~~

This module implements the NoCache and ShelveCache classes.
"""

import os
import tempfile
import shelve
from datetime import datetime


class NoCache(object):
    """
    A simple noop implementation for the cache. Any object implementing the same methods
    (duck typing) is acceptable as a cache backend.

    For example, python-memcached is compatible.
    """
    pass

    def set(self, key, val, ttl=0):
        pass

    def get(self, key):
        return None


class ShelveCache(object):
    """
    A cache implementation based on Shelve: https://docs.python.org/2/library/shelve.html.

    By default, it will be created with "filename" equal to the api domain name. If you want to
    run 2 processes using the same repository, you need to set a different file name to avoid
    concurrency problems.
    """
    def __init__(self, filename):
        self.filename = filename
        self.db = None

    def _init_db(self):
        if self.db is None:
            cache_dir = os.path.join(tempfile.mkdtemp(), "droplo-cache")
            if not os.path.exists(cache_dir):
                os.makedirs(cache_dir)
            self.db = shelve.open(os.path.join(cache_dir, self.filename))

    def set(self, key, val, ttl=0):
        self._init_db()
        if type(key) != str:
            key = key.encode('utf8')
        self.db[key] = {
            "val": val,
            "expire": ShelveCache.unix_time() + ttl
        }

    def get(self, key):
        self._init_db()
        if type(key) != str:
            key = key.encode('utf8')
        if key not in self.db:
            return None
        d = self.db[key]
        if d["expire"] < ShelveCache.unix_time():
            del self.db[key]
            return None
        else:
            return d["val"]

    @staticmethod
    def unix_time():
        epoch = datetime.utcfromtimestamp(0)
        delta = datetime.now() - epoch
        return delta.total_seconds()



import setuptools
import sys
import os
import re
from setuptools import setup
from setuptools.command.install import install

package = 'droplo'

with open('README.md') as f:
    long_description = f.read()


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]",
                     init_py, re.MULTILINE).group(1)


# python setup.py publish
if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    args = {'version': get_version(package)}
    print("Pushing tags to GitHub:")
    os.system("git tag -a %(version)s -m 'version %(version)s'" % args)
    os.system("git push --tags")
    os.system("git push")
    sys.exit()


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        version = get_version(package)
        tag = os.getenv('CIRCLE_TAG')

        if tag != version:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, version
            )
            sys.exit(info)


setup(
    name="droplo",
    version=get_version(package),
    url="https://github.com/droploio/droplo.py",

    author="Droplo Inc",
    author_email="contact@droplo.io",

    description="Official Python SDK for the Droplo content API",
    long_description=long_description,
    long_description_content_type='text/markdown',

    packages=setuptools.find_packages(),

    install_requires=['requests>=2.14.0'],

    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    test_suite='tests',
    tests_require=[],
    cmdclass={
        'verify': VerifyVersionCommand,
    }
)
